// ReadThis! Encrypted/Clear Text handling
//
// Copyright (C) 1999 by Schachar Levin <pixel.il@usa.net>.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

package ReadThis;

import Acme.Crypto.*;
import Acme.Crypto.CryptoUtils;
public class RTText
{
	protected String rtText; // Original Text
	private int begin,end;
	private String cipherAscData;
	private byte[] cipherRawData;
	private long KeyID;
	private String cipherType;
	private boolean encrypted;
	protected RTKey key;
	private void init()
	{
		cipherType=null;
		encrypted=false;
	}
	public void setKey(RTKey key)
	{
		this.key=key;
	}
	public String getCipherType()
	{
		return cipherType;
	}
	public boolean isEncrypted()
	{
		return encrypted;
	}
	public byte[] getRawData()
	{
		return cipherRawData;
	}
	public long getKeyID()
	{
		return key.getKeyID();
	}
	public long getTextKeyID()
	{
		return (KeyID&0xffffffff);
	}
	public void setText(String text)
	{
		init();
		this.rtText=text;
		KeyID=0;
		doText();
	}
	public void setCipherType(String type)
	{
		cipherType=type;
	}
	public RTText(String rtText)
	{
		this.rtText=rtText;
		init();
		doText();
	}
	private void doText()
	{
		int mark;
		String s;
		encrypted=false;
		cipherType="";
		KeyID=0;
		begin=rtText.indexOf("{RT"); // find start of encrypted part;
		if(begin==-1)
			return;
		end=rtText.indexOf("qq}"); // end of encrypted part
		if(end==-1)
			return;
		s=rtText.substring(begin+3,end+2); // remove header
		cipherType=s.substring(0,2); // cipher type
		s=s.substring(2,s.length()); // start of key ID
		mark=s.indexOf("qq")+2; // end of key ID
		KeyID=AscBin.toLong(s.substring(0,mark)); // key ID
		cipherRawData=AscBin.decode(s.substring(mark,s.length()));
		encrypted=true;
	}
	
	public boolean canEncrypt()
	{
		if(encrypted)
			return false;
		if(key.getKeyID()==0)
			return false;
		return true;
	}
	public boolean canDecrypt()
	{
		if(!encrypted)
			return false;
		if(KeyID!=key.getKeyID())
			return false;
		return true;
	}
	
	public String decrypt()
	{
		int size;
		byte[] clearRawData;
		byte[] clearText;
		byte[] key;

		if(!canDecrypt())
			return null;
		size=cipherRawData.length;
		clearRawData=new byte[size];
		key=this.key.getKey();

		if(cipherType.equals("XR"))
		{
			
			for(int i=0;i<size;i++)
			{
				clearRawData[i]=(byte)(cipherRawData[i]^key[i%16]);
			}
		}else
			if(cipherType.equals("ID"))
			{
				IdeaCipher id=new IdeaCipher(key);
				for(int i=0;i<size;i+=8)
					id.decrypt(cipherRawData,i,clearRawData,i);
				
			}else
				return null;
		size=((clearRawData[0]&0xff)<<8)+(clearRawData[1]&0xff);
		clearText=new byte[size];
		for(int i=0;i<size;i++)
		{
		clearText[i]=clearRawData[i+2];
		}
		String s="";
		if(begin>0)
			s=rtText.substring(0,begin);
		s=s+(new String(clearText));
		if(end+3<rtText.length())
			s=s+rtText.substring(end+3,rtText.length());
		return new String(s);
	}
	
	public String encryptToString()
	{
		String ct=AscBin.encode(encrypt());
		String ki=AscBin.fromLong(key.getKeyID());
		return new String("{RT"+cipherType+ki+ct+"}");
	}
	public byte[] encrypt()
	{
		int size;
		byte[] key;
        byte[] clearRawData;
		if(!canEncrypt())
			return null;
		size=rtText.length()+2;
		if((size%8)!=0) size=((size/8)+1)*8;
		cipherRawData=new byte[size];
		clearRawData=new byte[size];
		CryptoUtils.randomBlock(cipherRawData,0,size);
		for(int i=0;i<rtText.length();i++)
		{
			clearRawData[i+2]=rtText.getBytes()[i];
		}
		clearRawData[0]=(byte)((rtText.length()>>8)&0xff);
		clearRawData[1]=(byte)(rtText.length()&0xff);
		key=this.key.getKey();

		if(cipherType.equals("XR"))
		{
			
			for(int i=0;i<size;i++)
			{
				cipherRawData[i]=(byte)((clearRawData[i]^key[i%16])&0xff);
			}
		}else
			if(cipherType.equals("ID"))
			{
				IdeaCipher id=new IdeaCipher(key);
				for(int i=0;i<size;i+=8)
					id.encrypt(clearRawData,i,cipherRawData,i);
				
			}else
				return null;
	
		
		
		return cipherRawData;
	}
}

